diff --git a/addons/account/account_invoice.py b/addons/account/account_invoice.py
index eb43830f8f69f3c9aec2c08de74a1cb54985ec54..96d63c545c6065bf992092aa3d2e5d31fde16466 100644
--- a/addons/account/account_invoice.py
+++ b/addons/account/account_invoice.py
@@ -295,7 +295,8 @@ class account_invoice(osv.osv):
             },
             multi='all'),
         'currency_id': fields.many2one('res.currency', 'Currency', required=True, readonly=True, states={'draft':[('readonly',False)]}, track_visibility='always'),
-        'journal_id': fields.many2one('account.journal', 'Journal', required=True, readonly=True, states={'draft':[('readonly',False)]}),
+        'journal_id': fields.many2one('account.journal', 'Journal', required=True, readonly=True, states={'draft':[('readonly',False)]},
+                                      domain="[('type', 'in', {'out_invoice': ['sale'], 'out_refund': ['sale_refund'], 'in_refund': ['purchase_refund'], 'in_invoice': ['purchase']}.get(type, [])), ('company_id', '=', company_id)]"),
         'company_id': fields.many2one('res.company', 'Company', required=True, change_default=True, readonly=True, states={'draft':[('readonly',False)]}),
         'check_total': fields.float('Verification Total', digits_compute=dp.get_precision('Account'), readonly=True, states={'draft':[('readonly',False)]}),
         'reconciled': fields.function(_reconciled, string='Paid/Reconciled', type='boolean',
diff --git a/addons/account/views/report_agedpartnerbalance.xml b/addons/account/views/report_agedpartnerbalance.xml
index 019f32a35deeeb45323280dbe4190a138e50e179..c1f9047e3309e0ee73d762b2498dc67700d503c5 100644
--- a/addons/account/views/report_agedpartnerbalance.xml
+++ b/addons/account/views/report_agedpartnerbalance.xml
@@ -99,7 +99,7 @@
                         </tr>
                         <tr t-foreach="get_lines_with_out_partner(data['form'])" t-as="not_partner">
                             <td>
-                                <span t-esc="partner['name']"/>
+                                <span t-esc="not_partner['name']"/>
                             </td>
                             <td class="text-right">
                                 <span t-esc="formatLang(not_partner['direction'], currency_obj=res_company.currency_id)"/>
diff --git a/addons/account_analytic_analysis/account_analytic_analysis.py b/addons/account_analytic_analysis/account_analytic_analysis.py
index 519660fb708a605532a41249183274882f0eab3d..9550f77bcab5de52d9cb4858e6580cf18d3c26de 100644
--- a/addons/account_analytic_analysis/account_analytic_analysis.py
+++ b/addons/account_analytic_analysis/account_analytic_analysis.py
@@ -22,7 +22,6 @@ from dateutil.relativedelta import relativedelta
 import datetime
 import logging
 import time
-import traceback
 
 from openerp.osv import osv, fields
 from openerp.osv.orm import intersect, except_orm
@@ -73,6 +72,7 @@ class account_analytic_invoice_line(osv.osv):
 
         result = {}
         res = self.pool.get('product.product').browse(cr, uid, product, context=local_context)
+        price = False
         if price_unit is not False:
             price = price_unit
         elif pricelist_id:
@@ -746,29 +746,32 @@ class account_analytic_account(osv.osv):
             contract_ids = ids
         else:
             contract_ids = self.search(cr, uid, [('recurring_next_date','<=', current_date), ('state','=', 'open'), ('recurring_invoices','=', True), ('type', '=', 'contract')])
-        for contract in self.browse(cr, uid, contract_ids, context=context):
-            try:
-                invoice_values = self._prepare_invoice(cr, uid, contract, context=context)
-                invoice_ids.append(self.pool['account.invoice'].create(cr, uid, invoice_values, context=context))
-                next_date = datetime.datetime.strptime(contract.recurring_next_date or current_date, "%Y-%m-%d")
-                interval = contract.recurring_interval
-                if contract.recurring_rule_type == 'daily':
-                    new_date = next_date+relativedelta(days=+interval)
-                elif contract.recurring_rule_type == 'weekly':
-                    new_date = next_date+relativedelta(weeks=+interval)
-                elif contract.recurring_rule_type == 'monthly':
-                    new_date = next_date+relativedelta(months=+interval)
-                else:
-                    new_date = next_date+relativedelta(years=+interval)
-                self.write(cr, uid, [contract.id], {'recurring_next_date': new_date.strftime('%Y-%m-%d')}, context=context)
-                if automatic:
-                    cr.commit()
-            except Exception:
-                if automatic:
-                    cr.rollback()
-                    _logger.error(traceback.format_exc())
-                else:
-                    raise
+        if contract_ids:
+            cr.execute('SELECT company_id, array_agg(id) as ids FROM account_analytic_account WHERE id IN %s GROUP BY company_id', (tuple(contract_ids),))
+            for company_id, ids in cr.fetchall():
+                for contract in self.browse(cr, uid, ids, context=dict(context, company_id=company_id, force_company=company_id)):
+                    try:
+                        invoice_values = self._prepare_invoice(cr, uid, contract, context=context)
+                        invoice_ids.append(self.pool['account.invoice'].create(cr, uid, invoice_values, context=context))
+                        next_date = datetime.datetime.strptime(contract.recurring_next_date or current_date, "%Y-%m-%d")
+                        interval = contract.recurring_interval
+                        if contract.recurring_rule_type == 'daily':
+                            new_date = next_date+relativedelta(days=+interval)
+                        elif contract.recurring_rule_type == 'weekly':
+                            new_date = next_date+relativedelta(weeks=+interval)
+                        elif contract.recurring_rule_type == 'monthly':
+                            new_date = next_date+relativedelta(months=+interval)
+                        else:
+                            new_date = next_date+relativedelta(years=+interval)
+                        self.write(cr, uid, [contract.id], {'recurring_next_date': new_date.strftime('%Y-%m-%d')}, context=context)
+                        if automatic:
+                            cr.commit()
+                    except Exception:
+                        if automatic:
+                            cr.rollback()
+                            _logger.exception('Fail to create recurring invoice for contract %s', contract.code)
+                        else:
+                            raise
         return invoice_ids
 
 class account_analytic_account_summary_user(osv.osv):
diff --git a/addons/account_voucher/voucher_sales_purchase_view.xml b/addons/account_voucher/voucher_sales_purchase_view.xml
index d0064d6336c8c55a6e92af1bd2d8f1e71ab0df6e..f8c2d65f63bf8b6cca56cf2dc9e3f4fb01150303 100644
--- a/addons/account_voucher/voucher_sales_purchase_view.xml
+++ b/addons/account_voucher/voucher_sales_purchase_view.xml
@@ -248,7 +248,7 @@
                         <page string="Bill Information">
                             <field name="line_dr_ids" on_change="onchange_price(line_dr_ids, tax_id, partner_id)" context="{'journal_id':journal_id,'partner_id':partner_id}">
                                 <tree string="Expense Lines" editable="bottom">
-                                    <field name="account_id" widget="selection" domain="[('user_type.report_type','=','expense'), ('type','!=','view')]"/>
+                                    <field name="account_id" domain="[('user_type.report_type','=','expense'), ('type','!=','view')]"/>
                                     <field name="name"/>
                                     <field name="amount"/>
                                     <field name="account_analytic_id" groups="analytic.group_analytic_accounting"/>
diff --git a/addons/auth_oauth/controllers/main.py b/addons/auth_oauth/controllers/main.py
index 99f134a5cbd3ad7155b8770fdc0596ca10bb169b..7652420d7c6e59f9ab93e50fb4f7bb949d4f87b9 100644
--- a/addons/auth_oauth/controllers/main.py
+++ b/addons/auth_oauth/controllers/main.py
@@ -74,7 +74,7 @@ class OAuthLogin(Home):
         state = dict(
             d=request.session.db,
             p=provider['id'],
-            r=redirect,
+            r=werkzeug.url_quote_plus(redirect),
         )
         token = request.params.get('token')
         if token:
@@ -143,7 +143,7 @@ class OAuthController(http.Controller):
                 cr.commit()
                 action = state.get('a')
                 menu = state.get('m')
-                redirect = state.get('r')
+                redirect = werkzeug.url_unquote_plus(state['r']) if state.get('r') else False
                 url = '/web'
                 if redirect:
                     url = redirect
diff --git a/addons/calendar/static/src/js/base_calendar.js b/addons/calendar/static/src/js/base_calendar.js
index eed958e20d9ac1406f90168d7153e8cac65d89dd..37669840818c75fe1b8314dadbfbf0edadf43410 100644
--- a/addons/calendar/static/src/js/base_calendar.js
+++ b/addons/calendar/static/src/js/base_calendar.js
@@ -102,7 +102,7 @@ openerp.calendar = function(instance) {
             var self = this;
             var action_url = '';
 
-            action_url = _.str.sprintf('/?db=%s#id=%s&view_type=form&model=calendar.event', db, meeting_id);
+            action_url = _.str.sprintf('/web?db=%s#id=%s&view_type=form&model=calendar.event', db, meeting_id);
             
             var reload_page = function(){
                 return location.replace(action_url);
diff --git a/addons/event/report/report_event_registration.py b/addons/event/report/report_event_registration.py
index e76b5c60d1fe85cf32432630c4c2aa3aacd8ac82..c4a4dfd6b2d4a5c514a07a16632fd8a8ad5b99e2 100644
--- a/addons/event/report/report_event_registration.py
+++ b/addons/event/report/report_event_registration.py
@@ -32,7 +32,7 @@ class report_event_registration(osv.osv):
         'draft_state': fields.integer(' # No of Draft Registrations', size=20),
         'confirm_state': fields.integer(' # No of Confirmed Registrations', size=20),
         'seats_max': fields.integer('Max Seats'),
-        'nbevent': fields.integer('Number Of Events'),
+        'nbevent': fields.integer('Number of Registrations'),
         'event_type': fields.many2one('event.type', 'Event Type'),
         'registration_state': fields.selection([('draft', 'Draft'), ('confirm', 'Confirmed'), ('done', 'Attended'), ('cancel', 'Cancelled')], 'Registration State', readonly=True, required=True),
         'event_state': fields.selection([('draft', 'Draft'), ('confirm', 'Confirmed'), ('done', 'Done'), ('cancel', 'Cancelled')], 'Event State', readonly=True, required=True),
@@ -59,7 +59,7 @@ class report_event_registration(osv.osv):
                 r.name AS name_registration,
                 e.company_id AS company_id,
                 e.date_begin AS event_date,
-                count(e.id) AS nbevent,
+                count(r.id) AS nbevent,
                 CASE WHEN r.state IN ('draft') THEN r.nb_register ELSE 0 END AS draft_state,
                 CASE WHEN r.state IN ('open','done') THEN r.nb_register ELSE 0 END AS confirm_state,
                 e.type AS event_type,
diff --git a/addons/event/static/src/css/event.css b/addons/event/static/src/css/event.css
index ff5bfaba19b4e792f46b387cb9b2ce743bfae7f0..53c03cc03f007b3c2ea2b32dcfaceed7b8049b11 100644
--- a/addons/event/static/src/css/event.css
+++ b/addons/event/static/src/css/event.css
@@ -1,7 +1,7 @@
 .oe_event_date{
   border-top-left-radius:3px;
   border-top-right-radius:3px;
-  font-size: 48px;
+  font-size: 36px;
   height: auto;
   font-weight: bold;
   text-align: center;
diff --git a/addons/event_sale/event_sale.py b/addons/event_sale/event_sale.py
index ed99e6a39cc6ac7055cc8aae48dc0f8896c4db31..69ad0cb6af6ac1512e0356105b616eef49b1ec4b 100644
--- a/addons/event_sale/event_sale.py
+++ b/addons/event_sale/event_sale.py
@@ -245,7 +245,8 @@ class event_ticket(osv.osv):
     ]
 
     def onchange_product_id(self, cr, uid, ids, product_id=False, context=None):
-        return {'value': {'price': self.pool.get("product.product").browse(cr, uid, product_id).list_price or 0}}
+        price = self.pool.get("product.product").browse(cr, uid, product_id).list_price if product_id else 0
+        return {'value': {'price': price}}
 
 
 class event_registration(osv.osv):
diff --git a/addons/gamification/models/challenge.py b/addons/gamification/models/challenge.py
index ddc0833629375e03aed86946ff7369dd2ab29cca..faed62f8da92bac673535297e21307a2d977e703 100644
--- a/addons/gamification/models/challenge.py
+++ b/addons/gamification/models/challenge.py
@@ -58,7 +58,7 @@ def start_end_date_for_period(period, default_start_date=False, default_end_date
         end_date = default_end_date
 
     if start_date and end_date:
-        return (start_date.strftime(DF), end_date.strftime(DF))
+        return (datetime.strftime(start_date, DF), datetime.strftime(end_date, DF))
     else:
         return (start_date, end_date)
 
diff --git a/addons/google_calendar/google_calendar.py b/addons/google_calendar/google_calendar.py
index 543c81977aaefed971e14feb72d1bbe689dd929c..dd1d45c05fe32519ea2877ca8d9c6757a189c92d 100644
--- a/addons/google_calendar/google_calendar.py
+++ b/addons/google_calendar/google_calendar.py
@@ -699,7 +699,7 @@ class google_calendar(osv.AbstractModel):
         for att in att_obj.browse(cr, uid, my_att_ids, context=context):
             event = att.event_id
 
-            base_event_id = att.google_internal_event_id.split('_')[0]
+            base_event_id = att.google_internal_event_id.rsplit('_', 1)[0]
 
             if base_event_id not in event_to_synchronize:
                 event_to_synchronize[base_event_id] = {}
@@ -721,7 +721,7 @@ class google_calendar(osv.AbstractModel):
 
         for event in all_event_from_google.values():
             event_id = event.get('id')
-            base_event_id = event_id.split('_')[0]
+            base_event_id = event_id.rsplit('_', 1)[0]
 
             if base_event_id not in event_to_synchronize:
                 event_to_synchronize[base_event_id] = {}
@@ -786,7 +786,7 @@ class google_calendar(osv.AbstractModel):
                     if actSrc == 'OE':
                         self.delete_an_event(cr, uid, current_event[0], context=context)
                     elif actSrc == 'GG':
-                        new_google_event_id = event.GG.event['id'].split('_')[1]
+                        new_google_event_id = event.GG.event['id'].rsplit('_', 1)[1]
                         if 'T' in new_google_event_id:
                             new_google_event_id = new_google_event_id.replace('T', '')[:-1]
                         else:
@@ -795,7 +795,8 @@ class google_calendar(osv.AbstractModel):
                         if event.GG.status:
                             parent_event = {}
                             if not event_to_synchronize[base_event][0][1].OE.event_id:
-                                event_to_synchronize[base_event][0][1].OE.event_id = att_obj.search_read(cr, uid, [('google_internal_event_id', '=', event.GG.event['id'].split('_')[0])], ['event_id'], context=context_novirtual)[0].get('event_id')[0]
+                                main_ev = att_obj.search_read(cr, uid, [('google_internal_event_id', '=', event.GG.event['id'].rsplit('_', 1)[0])], fields=['event_id'], context=context_novirtual)
+                                event_to_synchronize[base_event][0][1].OE.event_id = main_ev[0].get('event_id')[0]
 
                             parent_event['id'] = "%s-%s" % (event_to_synchronize[base_event][0][1].OE.event_id, new_google_event_id)
                             res = self.update_from_google(cr, uid, parent_event, event.GG.event, "copy", context)
diff --git a/addons/hr/hr.py b/addons/hr/hr.py
index 85366adfe3b8cf7c9263f7dcc34a642590c6a97f..f4719f572f0cda7bea49b5eb98de86b78bf8d069 100644
--- a/addons/hr/hr.py
+++ b/addons/hr/hr.py
@@ -225,7 +225,7 @@ class hr_employee(osv.osv):
                  "resized as a 128x128px image, with aspect ratio preserved. "\
                  "Use this field in form views or some kanban views."),
         'image_small': fields.function(_get_image, fnct_inv=_set_image,
-            string="Smal-sized photo", type="binary", multi="_get_image",
+            string="Small-sized photo", type="binary", multi="_get_image",
             store = {
                 'hr.employee': (lambda self, cr, uid, ids, c={}: ids, ['image'], 10),
             },
diff --git a/addons/hr/hr_view.xml b/addons/hr/hr_view.xml
index 09e7373384e5b7eea0f97a71b4323b185667a170..0c35670640742e14c535619c0d3830098ae2039f 100644
--- a/addons/hr/hr_view.xml
+++ b/addons/hr/hr_view.xml
@@ -40,7 +40,9 @@
                             <page string="Public Information">
                                 <group>
                                     <group string="Contact Information">
-                                        <field name="address_id" on_change="onchange_address_id(address_id)" context="{'show_address': 1}" options='{"always_reload": True, "highlight_first_line": True}'/>
+                                        <field name="address_id" on_change="onchange_address_id(address_id)"
+                                            context="{'show_address': 1, 'default_customer': False}"
+                                            options='{"always_reload": True, "highlight_first_line": True}'/>
                                         <field name="mobile_phone"/>
                                         <field name="work_location"/>
                                     </group>
@@ -68,7 +70,9 @@
                                         <field name="otherid" groups="base.group_hr_user"/>
                                     </group>
                                     <group string="Contact Information">
-                                        <field name="address_home_id" context="{'show_address': 1}" options='{"always_reload": True, "highlight_first_line": True}'/>
+                                        <field name="address_home_id"
+                                            context="{'show_address': 1, 'default_customer': False}"
+                                            options='{"always_reload": True, "highlight_first_line": True}'/>
                                     </group>
                                     <group string="Status">
                                         <field name="gender"/>
diff --git a/addons/hr_holidays/hr_holidays.py b/addons/hr_holidays/hr_holidays.py
index 3d467e0b065b4482617bc98b0efcea012a048953..ed25b75aa836889da7c575992ac9c29737a74d7f 100644
--- a/addons/hr_holidays/hr_holidays.py
+++ b/addons/hr_holidays/hr_holidays.py
@@ -99,7 +99,7 @@ class hr_holidays_status(osv.osv):
         for record in self.browse(cr, uid, ids, context=context):
             name = record.name
             if not record.limit:
-                name = name + ('  (%d/%d)' % (record.leaves_taken or 0.0, record.max_leaves or 0.0))
+                name = name + ('  (%g/%g)' % (record.leaves_taken or 0.0, record.max_leaves or 0.0))
             res.append((record.id, name))
         return res
 
diff --git a/addons/l10n_be_invoice_bba/invoice.py b/addons/l10n_be_invoice_bba/invoice.py
index 854593a87cc391532f1c63bb425b8a04e7a3af28..499e6e6c4ac1057bd74d5f1e6e00b21d96c44daa 100644
--- a/addons/l10n_be_invoice_bba/invoice.py
+++ b/addons/l10n_be_invoice_bba/invoice.py
@@ -141,7 +141,7 @@ class account_invoice(osv.osv):
                 elif algorithm == 'random':
                     if not self.check_bbacomm(reference):
                         base = random.randint(1, 9999999999)
-                        bbacomm = str(base).rjust(7, '0')
+                        bbacomm = str(base).rjust(10, '0')
                         base = int(bbacomm)
                         mod = base % 97 or 97
                         mod = str(mod).rjust(2, '0')
diff --git a/addons/mail/mail_group.py b/addons/mail/mail_group.py
index 4ae47a9677476547fca104b4fb94248725f3de06..186787c121a178331fed15368128b67ccff5ec41 100644
--- a/addons/mail/mail_group.py
+++ b/addons/mail/mail_group.py
@@ -211,3 +211,16 @@ class mail_group(osv.Model):
             return []
         else:
             return super(mail_group, self).get_suggested_thread(cr, uid, removed_suggested_threads, context)
+
+    def message_get_email_values(self, cr, uid, id, notif_mail=None, context=None):
+        res = super(mail_group, self).message_get_email_values(cr, uid, id, notif_mail=notif_mail, context=context)
+        group = self.browse(cr, uid, id, context=context)
+        res.update({
+            'headers': {
+                'Precedence': 'list',
+            }
+        })
+        if group.alias_domain:
+            res['headers']['List-Id'] = '%s.%s' % (group.alias_name, group.alias_domain)
+            res['headers']['List-Post'] = '<mailto:%s@%s>' % (group.alias_name, group.alias_domain)
+        return res
diff --git a/addons/mail/mail_mail.py b/addons/mail/mail_mail.py
index a481414047937bc7ea7b4137d02b1520bb82153b..cc12bde2436933d7746cf568d96abeae95bc48f7 100644
--- a/addons/mail/mail_mail.py
+++ b/addons/mail/mail_mail.py
@@ -204,12 +204,15 @@ class mail_mail(osv.Model):
         """
         body = self.send_get_mail_body(cr, uid, mail, partner=partner, context=context)
         body_alternative = tools.html2plaintext(body)
-        return {
+        res = {
             'body': body,
             'body_alternative': body_alternative,
             'subject': self.send_get_mail_subject(cr, uid, mail, partner=partner, context=context),
             'email_to': self.send_get_mail_to(cr, uid, mail, partner=partner, context=context),
         }
+        if mail.model and mail.res_id and self.pool.get(mail.model) and hasattr(self.pool[mail.model], 'message_get_email_values'):
+            res.update(self.pool[mail.model].message_get_email_values(cr, uid, mail.res_id, mail, context=context))
+        return res
 
     def send(self, cr, uid, ids, auto_commit=False, raise_exception=False, context=None):
         """ Sends the selected emails immediately, ignoring their current
@@ -268,6 +271,9 @@ class mail_mail(osv.Model):
                 # build an RFC2822 email.message.Message object and send it without queuing
                 res = None
                 for email in email_list:
+                    email_headers = dict(headers)
+                    if email.get('headers'):
+                        email_headers.update(email['headers'])
                     msg = ir_mail_server.build_email(
                         email_from=mail.email_from,
                         email_to=email.get('email_to'),
@@ -282,7 +288,7 @@ class mail_mail(osv.Model):
                         object_id=mail.res_id and ('%s-%s' % (mail.res_id, mail.model)),
                         subtype='html',
                         subtype_alternative='plain',
-                        headers=headers)
+                        headers=email_headers)
                     res = ir_mail_server.send_email(cr, uid, msg,
                                                     mail_server_id=mail.mail_server_id.id,
                                                     context=context)
diff --git a/addons/mail/mail_thread.py b/addons/mail/mail_thread.py
index dc2b98fdc93885a158ebda9eaf5b1c5b6530773a..76d3ab744d5d2a505809ae22636d82c705ae9cc1 100644
--- a/addons/mail/mail_thread.py
+++ b/addons/mail/mail_thread.py
@@ -34,6 +34,7 @@ import pytz
 import socket
 import time
 import xmlrpclib
+import re
 from email.message import Message
 from urllib import urlencode
 
@@ -48,6 +49,8 @@ from openerp.tools.translate import _
 _logger = logging.getLogger(__name__)
 
 
+mail_header_msgid_re = re.compile('<[^<>]+>')
+
 def decode_header(message, header, separator=' '):
     return separator.join(map(decode, filter(None, message.get_all(header, []))))
 
@@ -694,6 +697,16 @@ class mail_thread(osv.AbstractModel):
                 if record.alias_domain and record.alias_name else False
                 for record in self.browse(cr, SUPERUSER_ID, ids, context=context)]
 
+    def message_get_email_values(self, cr, uid, id, notif_mail=None, context=None):
+        """ Temporary method to create custom notification email values for a given
+        model and document. This should be better to have a headers field on
+        the mail.mail model, computed when creating the notification email, but
+        this cannot be done in a stable version.
+
+        TDE FIXME: rethink this ulgy thing. """
+        res = dict()
+        return res
+
     #------------------------------------------------------
     # Mail gateway
     #------------------------------------------------------
@@ -1301,13 +1314,13 @@ class mail_thread(osv.AbstractModel):
             msg_dict['date'] = stored_date.strftime(tools.DEFAULT_SERVER_DATETIME_FORMAT)
 
         if message.get('In-Reply-To'):
-            parent_ids = self.pool.get('mail.message').search(cr, uid, [('message_id', '=', decode(message['In-Reply-To']))])
+            parent_ids = self.pool.get('mail.message').search(cr, uid, [('message_id', '=', decode(message['In-Reply-To'].strip()))])
             if parent_ids:
                 msg_dict['parent_id'] = parent_ids[0]
 
         if message.get('References') and 'parent_id' not in msg_dict:
-            parent_ids = self.pool.get('mail.message').search(cr, uid, [('message_id', 'in',
-                                                                         [x.strip() for x in decode(message['References']).split()])])
+            msg_list =  mail_header_msgid_re.findall(decode(message['References']))
+            parent_ids = self.pool.get('mail.message').search(cr, uid, [('message_id', 'in', [x.strip() for x in msg_list])])
             if parent_ids:
                 msg_dict['parent_id'] = parent_ids[0]
 
diff --git a/addons/mail/wizard/mail_compose_message.py b/addons/mail/wizard/mail_compose_message.py
index cb068d566c101a8df4f5bd8e1d5567c0bc4e49d9..e14a8f3dfe5b59578b99e880d06baf61c7cd09b2 100644
--- a/addons/mail/wizard/mail_compose_message.py
+++ b/addons/mail/wizard/mail_compose_message.py
@@ -267,10 +267,7 @@ class mail_compose_message(osv.TransientModel):
             # mass mailing: rendering override wizard static values
             if mass_mail_mode and wizard.model:
                 # always keep a copy, reset record name (avoid browsing records)
-                mail_values.update(notification=True, record_name=False)
-                if hasattr(self.pool[wizard.model], 'message_new'):
-                    mail_values['model'] = wizard.model
-                    mail_values['res_id'] = res_id
+                mail_values.update(notification=True, model=wizard.model, res_id=res_id, record_name=False)
                 # auto deletion of mail_mail
                 if 'mail_auto_delete' in context:
                     mail_values['auto_delete'] = context.get('mail_auto_delete')
diff --git a/addons/mass_mailing/models/mail_mail.py b/addons/mass_mailing/models/mail_mail.py
index a67f88a2a2876a258db6645483ddd36aa8d15562..0e44399a256f0423c6797f19c021292f1697fdd6 100644
--- a/addons/mass_mailing/models/mail_mail.py
+++ b/addons/mass_mailing/models/mail_mail.py
@@ -84,7 +84,8 @@ class MailMail(osv.Model):
     def send_get_email_dict(self, cr, uid, mail, partner=None, context=None):
         res = super(MailMail, self).send_get_email_dict(cr, uid, mail, partner, context=context)
         if mail.mailing_id and res.get('body') and res.get('email_to'):
-            email_to = tools.email_split(res.get('email_to')[0])
+            emails = tools.email_split(res.get('email_to')[0])
+            email_to = emails and emails[0] or False
             unsubscribe_url = self._get_unsubscribe_url(cr, uid, mail, email_to, context=context)
             if unsubscribe_url:
                 res['body'] = tools.append_content_to_html(res['body'], unsubscribe_url, plaintext=False, container_tag='p')
diff --git a/addons/mass_mailing/views/mass_mailing.xml b/addons/mass_mailing/views/mass_mailing.xml
index 9806022498ae3726429110706eead33f54e6e8d5..2939ebc3711cf55abcbf261a1e5f2c3c2ebb13de 100644
--- a/addons/mass_mailing/views/mass_mailing.xml
+++ b/addons/mass_mailing/views/mass_mailing.xml
@@ -591,6 +591,7 @@
                     <field name="mail_mail_id"/>
                     <field name="message_id"/>
                     <field name="sent"/>
+                    <field name="exception"/>
                     <field name="opened"/>
                     <field name="replied"/>
                     <field name="bounced"/>
diff --git a/addons/mrp/mrp.py b/addons/mrp/mrp.py
index f5b6cf466a5f83c3842c3bb571e30345bc9a31ef..ca4109c1b9a29141922dd20bf94e420d0e533351 100644
--- a/addons/mrp/mrp.py
+++ b/addons/mrp/mrp.py
@@ -1071,8 +1071,8 @@ class mrp_production(osv.osv):
             return False
         # Take routing location as a Source Location.
         source_location_id = production.location_src_id.id
-        if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
-            source_location_id = production.bom_id.routing_id.location_id.id
+        if production.routing_id and production.routing_id.location_id:
+            source_location_id = production.routing_id.location_id.id
 
         destination_location_id = production.product_id.property_stock_production.id
         if not source_location_id:
diff --git a/addons/product/product.py b/addons/product/product.py
index eaed55c0b415a0988a1a981d8eb653773b473959..6fa0627db2c4733874023abb4d0ae89bb0e81688 100644
--- a/addons/product/product.py
+++ b/addons/product/product.py
@@ -678,6 +678,17 @@ class product_template(osv.osv):
         if not context or "create_product_product" not in context:
             self.create_variant_ids(cr, uid, [product_template_id], context=context)
         self._set_standard_price(cr, uid, product_template_id, vals.get('standard_price', 0.0), context=context)
+
+        # TODO: this is needed to set given values to first variant after creation
+        # these fields should be moved to product as lead to confusion
+        related_vals = {}
+        if vals.get('ean13'):
+            related_vals['ean13'] = vals['ean13']
+        if vals.get('default_code'):
+            related_vals['default_code'] = vals['default_code']
+        if related_vals:
+            self.write(cr, uid, product_template_id, related_vals, context=context)
+
         return product_template_id
 
     def write(self, cr, uid, ids, vals, context=None):
diff --git a/addons/product_email_template/models/invoice.py b/addons/product_email_template/models/invoice.py
index d9e82e4201257c69ad247bfa67c6306feb551fe9..d5876b510f10f3b91a7cd51c992ad755607b6c22 100644
--- a/addons/product_email_template/models/invoice.py
+++ b/addons/product_email_template/models/invoice.py
@@ -27,7 +27,7 @@ class account_invoice(osv.Model):
                     template_values = Composer.onchange_template_id(
                         cr, uid, composer_id, line.product_id.email_template_id.id, 'comment', 'account.invoice', invoice.id
                     )['value']
-                    template_values['attachment_ids'] = [(4, id) for id in template_values.get('attachment_ids', '[]')]
+                    template_values['attachment_ids'] = [(4, id) for id in template_values.get('attachment_ids', [])]
                     Composer.write(cr, uid, [composer_id], template_values, context=context)
                     Composer.send_mail(cr, uid, [composer_id], context=context)
         return True
diff --git a/addons/project/project_demo.xml b/addons/project/project_demo.xml
index 54f78ba8c8b4c3e717db21871f32dbf5d9fae0a1..695aa80af2a7114a40f50fad35c3c03fcef79a35 100644
--- a/addons/project/project_demo.xml
+++ b/addons/project/project_demo.xml
@@ -50,8 +50,8 @@
             <field name="user_id" ref="base.user_demo"/>
             <field name="alias_model">project.task</field>
             <field name="message_follower_ids" eval="[(6, 0, [
-                    ref('base.user_root'),
-                    ref('base.user_demo')])]"/>
+                    ref('base.partner_root'),
+                    ref('base.partner_demo')])]"/>
         </record>
 
         <!-- We assign after so that default values applies -->
diff --git a/addons/stock/stock_view.xml b/addons/stock/stock_view.xml
index fc7a94fc51f4f12bc253de663a08ca38d26d71b1..e2673b1ab1f67191b1bea43b3aeb06d623344d4e 100644
--- a/addons/stock/stock_view.xml
+++ b/addons/stock/stock_view.xml
@@ -749,7 +749,7 @@
                                 </tree>
                             </field>
 -->
-                            <field name="move_lines" context="{'address_in_id': partner_id, 'form_view_ref':'stock.view_move_picking_form', 'tree_view_ref':'view_move_picking_tree', 'default_picking_type_id': picking_type_id,'default_picking_id': active_id}"/>
+                            <field name="move_lines" context="{'address_in_id': partner_id, 'form_view_ref':'stock.view_move_picking_form', 'tree_view_ref':'stock.view_move_picking_tree', 'default_picking_type_id': picking_type_id,'default_picking_id': active_id}"/>
                             <field name="pack_operation_exist" invisible="1"/>
                             <field name="note" placeholder="Add an internal note..." class="oe_inline"/>
                         </page>
@@ -1288,7 +1288,7 @@
             <field name="type">ir.actions.act_window</field>
             <field name="view_type">form</field>
             <field name="view_mode">tree,form</field>
-            <field name="domain" eval="['|','&amp;',('picking_id','=',False),('location_dest_id.usage', 'in', ['customer','supplier']),'&amp;',('picking_id','!=',False),('picking_id.picking_type_id.code','=','outgoing')]"/>
+            <field name="domain" eval="[('picking_id.picking_type_id.code','=','incoming'), ('location_id.usage','!=','internal'), ('location_dest_id.usage', '=', 'internal')]"/>
             <field name="view_id" ref="view_move_tree_reception_picking"/>
             <field name="context">{'product_receive': True, 'search_default_future': True}</field>
             <field name="help" type="html">
diff --git a/addons/web/static/src/css/base.css b/addons/web/static/src/css/base.css
index b234e2738dfaef9e3b32c12b1b4fed99b2a11e47..9988e7eae746a1e680021884532e0717e63acf97 100644
--- a/addons/web/static/src/css/base.css
+++ b/addons/web/static/src/css/base.css
@@ -2373,7 +2373,7 @@
 }
 .openerp .oe_fileupload .oe_add button.oe_attach .oe_e {
   position: relative;
-  top: -1px;
+  top: -10px;
   left: -9px;
 }
 .openerp .oe_fileupload .oe_add input.oe_form_binary_file {
@@ -3324,6 +3324,9 @@ body.oe_single_form .oe_single_form_container {
 .openerp_ie ul.oe_form_status li.oe_active > .arrow span, .openerp_ie ul.oe_form_status_clickable li.oe_active > .arrow span {
   background-color: #729fcf !important;
 }
+}
+.openerp_ie .oe_webclient {
+  height: auto !important;
 
 @media print {
   .openerp {
diff --git a/addons/web/static/src/css/base.sass b/addons/web/static/src/css/base.sass
index 093ebf6cb0bcc096b0416efefbbc61ee0f95a80e..b85346a2c8f6fe80b8fb6bfa9616e3ea615d7f78 100644
--- a/addons/web/static/src/css/base.sass
+++ b/addons/web/static/src/css/base.sass
@@ -1939,7 +1939,7 @@ $sheet-padding: 16px
                 text-shadow: none
                 .oe_e
                     position: relative
-                    top: -1px
+                    top: -10px
                     left: -9px                    
             input.oe_form_binary_file
                 display: inline-block
@@ -2691,6 +2691,8 @@ body.oe_single_form
             > .arrow span
                 background-color: #729fcf !important
 
+    .oe_webclient
+        height: auto !important
 // }}}
 
 // @media print {{{
diff --git a/addons/web/static/src/js/formats.js b/addons/web/static/src/js/formats.js
index 3c32dbe04260cd21ca2be9dac2e290a5b41e6ed4..f08dc31021a953a26d82ff4e8a3acdc87503fd7b 100644
--- a/addons/web/static/src/js/formats.js
+++ b/addons/web/static/src/js/formats.js
@@ -233,7 +233,8 @@ instance.web.parse_value = function (value, descriptor, value_if_empty) {
                 value = value.replace(instance.web._t.database.parameters.thousands_sep, "");
             } while(tmp !== value);
             tmp = Number(value);
-            if (isNaN(tmp))
+            // do not accept not numbers or float values
+            if (isNaN(tmp) || tmp % 1)
                 throw new Error(_.str.sprintf(_t("'%s' is not a correct integer"), value));
             return tmp;
         case 'float':
@@ -268,6 +269,11 @@ instance.web.parse_value = function (value, descriptor, value_if_empty) {
         case 'datetime':
             var datetime = Date.parseExact(
                     value, (date_pattern + ' ' + time_pattern));
+            if (datetime !== null)
+                return instance.web.datetime_to_str(datetime);
+            datetime = Date.parseExact(value.replace(/\d+/g, function(m){
+                return m.length === 1 ? "0" + m : m ;
+            }), (date_pattern + ' ' + time_pattern));
             if (datetime !== null)
                 return instance.web.datetime_to_str(datetime);
             datetime = Date.parse(value);
@@ -276,6 +282,11 @@ instance.web.parse_value = function (value, descriptor, value_if_empty) {
             throw new Error(_.str.sprintf(_t("'%s' is not a correct datetime"), value));
         case 'date':
             var date = Date.parseExact(value, date_pattern);
+            if (date !== null)
+                return instance.web.date_to_str(date);
+            date = Date.parseExact(value.replace(/\d+/g, function(m){
+                return m.length === 1 ? "0" + m : m ;
+            }), date_pattern);
             if (date !== null)
                 return instance.web.date_to_str(date);
             date = Date.parse(value);
diff --git a/addons/web/static/src/js/search.js b/addons/web/static/src/js/search.js
index 5653447566b9e4066a6039c147cc9fb1dbccfa9a..ac441d8eed676af35037daf88fdbc2e11c5ee151 100644
--- a/addons/web/static/src/js/search.js
+++ b/addons/web/static/src/js/search.js
@@ -346,11 +346,11 @@ instance.web.SearchView = instance.web.Widget.extend(/** @lends instance.web.Sea
         'keydown .oe_searchview_input, .oe_searchview_facet': function (e) {
             switch(e.which) {
             case $.ui.keyCode.LEFT:
-                this.focusPreceding(this);
+                this.focusPreceding(e.target);
                 e.preventDefault();
                 break;
             case $.ui.keyCode.RIGHT:
-                this.focusFollowing(this);
+                this.focusFollowing(e.target);
                 e.preventDefault();
                 break;
             }
diff --git a/addons/web/static/src/js/view_form.js b/addons/web/static/src/js/view_form.js
index e276a5e3995196020dab3c6ad7a5daf93dc83b47..cb0b2e8ea80e6921308276f0c477f7a4a09fc437 100644
--- a/addons/web/static/src/js/view_form.js
+++ b/addons/web/static/src/js/view_form.js
@@ -2632,6 +2632,7 @@ instance.web.DateTimeWidget = instance.web.Widget.extend({
     type_of_date: "datetime",
     events: {
         'change .oe_datepicker_master': 'change_datetime',
+        'keypress .oe_datepicker_master': 'change_datetime',
     },
     init: function(parent) {
         this._super(parent);
@@ -2750,8 +2751,8 @@ instance.web.DateTimeWidget = instance.web.Widget.extend({
     format_client: function(v) {
         return instance.web.format_value(v, {"widget": this.type_of_date});
     },
-    change_datetime: function() {
-        if (this.is_valid_()) {
+    change_datetime: function(e) {
+        if ((e.type !== "keypress" || e.which === 13) && this.is_valid_()) {
             this.set_value_from_ui_();
             this.trigger("datetime_changed");
         }
diff --git a/addons/web/static/src/js/view_list_editable.js b/addons/web/static/src/js/view_list_editable.js
index e2c63eb191ac59d333e4451f396fdbeaa17bc0da..ea2c28d9c3f924a5b8e5ac1ac4a9f727d5ff93aa 100644
--- a/addons/web/static/src/js/view_list_editable.js
+++ b/addons/web/static/src/js/view_list_editable.js
@@ -130,15 +130,7 @@
             if (this.editable()) {
                 this.$el.find('table:first').show();
                 this.$el.find('.oe_view_nocontent').remove();
-                this.start_edition().then(function(){
-                    var fields = self.editor.form.fields;
-                    self.editor.form.fields_order.some(function(field){
-                        if (fields[field].$el.is(':visible')){
-                            fields[field].$el.find("input").select();
-                            return true;
-                        }
-                    });
-                });
+                this.start_edition();
             } else {
                 this._super();
             }
@@ -243,6 +235,7 @@
             return this.ensure_saved().then(function () {
                 var $recordRow = self.groups.get_row_for(record);
                 var cells = self.get_cells_for($recordRow);
+                var fields = {};
                 self.fields_for_resize.splice(0, self.fields_for_resize.length);
                 return self.with_event('edit', {
                     record: record.attributes,
@@ -256,10 +249,16 @@
 
                         // FIXME: need better way to get the field back from bubbling (delegated) DOM events somehow
                         field.$el.attr('data-fieldname', field_name);
+                        fields[field_name] = field;
                         self.fields_for_resize.push({field: field, cell: cell});
                     }, options).then(function () {
                         $recordRow.addClass('oe_edition');
                         self.resize_fields();
+                        var focus_field = options && options.focus_field ? options.focus_field : undefined;
+                        if (!focus_field){
+                            focus_field = _.find(self.editor.form.fields_order, function(field){ return fields[field] && fields[field].$el.is(':visible:has(input)'); });
+                        }
+                        if (focus_field) fields[focus_field].$el.find('input').select();
                         return record.attributes;
                     });
                 }).fail(function () {
@@ -749,31 +748,6 @@
             throw new Error("is_editing's state filter must be either `new` or" +
                             " `edit` if provided");
         },
-        _focus_setup: function (focus_field) {
-            var form = this.form;
-
-            var field;
-            // If a field to focus was specified
-            if (focus_field
-                    // Is actually in the form
-                    && (field = form.fields[focus_field])
-                    // And is visible
-                    && field.$el.is(':visible')) {
-                // focus it
-                field.focus();
-                return;
-            }
-
-            _(form.fields_order).detect(function (name) {
-                // look for first visible field in fields_order, focus it
-                var field = form.fields[name];
-                if (!field.$el.is(':visible')) {
-                    return false;
-                }
-                // Stop as soon as a field got focused
-                return field.focus() !== false;
-            });
-        },
         edit: function (record, configureField, options) {
             // TODO: specify sequence of edit calls
             var self = this;
@@ -788,7 +762,6 @@
                 _(form.fields).each(function (field, name) {
                     configureField(name, field);
                 });
-                self._focus_setup(options && options.focus_field);
                 return form;
             });
         },
diff --git a/addons/web_calendar/static/src/js/web_calendar.js b/addons/web_calendar/static/src/js/web_calendar.js
index 23b11254b53a751807104972badc2a1e9b96578d..a7cff99f43654909b7e9b66d2898aa858ed14951 100644
--- a/addons/web_calendar/static/src/js/web_calendar.js
+++ b/addons/web_calendar/static/src/js/web_calendar.js
@@ -218,7 +218,12 @@ openerp.web_calendar = function(instance) {
                 this.info_fields.push(fv.arch.children[fld].attrs.name);
             }
 
-            return (new instance.web.Model(this.dataset.model))
+            var edit_check = new instance.web.Model(this.dataset.model)
+                .call("check_access_rights", ["write", false])
+                .then(function (write_right) {
+                    self.write_right = write_right;
+                });
+            var init = new instance.web.Model(this.dataset.model)
                 .call("check_access_rights", ["create", false])
                 .then(function (create_right) {
                     self.create_right = create_right;
@@ -228,6 +233,7 @@ openerp.web_calendar = function(instance) {
                         self.ready.resolve();
                     });
                 });
+            return $.when(edit_check, init);
         },
 
         get_fc_init_options: function () {
@@ -841,7 +847,11 @@ openerp.web_calendar = function(instance) {
             if (! this.open_popup_action) {
                 var index = this.dataset.get_id_index(id);
                 this.dataset.index = index;
-                this.do_switch_view('form', null, { mode: "edit" });
+                if (this.write_right) {
+                    this.do_switch_view('form', null, { mode: "edit" });
+                } else {
+                    this.do_switch_view('form', null, { mode: "view" });
+                }
             }
             else {
                 var pop = new instance.web.form.FormOpenPopup(this);
diff --git a/addons/website/static/src/js/website.editor.js b/addons/website/static/src/js/website.editor.js
index 601d3016068d3dc56dac4cd99218207351633133..8cd78377a909d9d709964e5c9139aae875b4c30e 100644
--- a/addons/website/static/src/js/website.editor.js
+++ b/addons/website/static/src/js/website.editor.js
@@ -469,7 +469,7 @@
                     }
                 );
             });
-            menu.on('click', 'a[data-action!=ace]', function (event) {
+            menu.on('click', 'a[data-view-id]', function (event) {
                 var view_id = $(event.currentTarget).data('view-id');
                 return openerp.jsonRpc('/web/dataset/call_kw', 'call', {
                     model: 'ir.ui.view',
@@ -1516,7 +1516,7 @@
                 url: this.link
             });
             this.media.renameNode("img");
-            this.media.$.attributes.src = this.link;
+            $(this.media).attr('src', this.link);
             return this._super();
         },
         clear: function () {
@@ -2000,6 +2000,11 @@
         //       a/@href, ...)
         _(mutations).chain()
             .filter(function (m) {
+                // ignore any SVG target, these blokes are like weird mon
+                if (m.target && m.target instanceof SVGElement) {
+                    return false;
+                }
+
                 // ignore any change related to mundane image-edit-button
                 if (m.target && m.target.className
                         && m.target.className.indexOf('image-edit-button') !== -1) {
diff --git a/addons/website/static/src/js/website.mobile.js b/addons/website/static/src/js/website.mobile.js
index 98d9bdc957acb25dc4b9dc4f4a503e4402b37c2c..1dbe8a1ece9252f5c33bf8a6319543dba48698f5 100644
--- a/addons/website/static/src/js/website.mobile.js
+++ b/addons/website/static/src/js/website.mobile.js
@@ -20,7 +20,7 @@
             if (!window.location.origin) { // fix for ie9
                 window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port: '');
             }
-            document.getElementById("mobile-viewport").src = window.location.origin + window.location.pathname + "#mobile-preview";
+            document.getElementById("mobile-viewport").src = window.location.origin + window.location.pathname + window.location.search + "#mobile-preview";
             this.$el.modal();
         },
         destroy: function () {
diff --git a/addons/website_forum/controllers/main.py b/addons/website_forum/controllers/main.py
index e13564a6a5e92b2c8da1e79c3654ada53f5047cb..aa4123d039af43f29c6f31b435ec9145702c2c66 100644
--- a/addons/website_forum/controllers/main.py
+++ b/addons/website_forum/controllers/main.py
@@ -120,7 +120,7 @@ class WebsiteForum(http.Controller):
 
         question_count = Post.search(cr, uid, domain, count=True, context=context)
         if tag:
-            url = "/forum/%s/%s/questions" % (slug(forum), slug(tag))
+            url = "/forum/%s/tag/%s/questions" % (slug(forum), slug(tag))
         else:
             url = "/forum/%s" % slug(forum)
 
diff --git a/addons/website_forum/static/src/js/website_forum.js b/addons/website_forum/static/src/js/website_forum.js
index 9d4e9e587c952dea837b05afe6b45051c6ef329a..5db211ab9aa59f93b29c27220166a093b4d55db2 100644
--- a/addons/website_forum/static/src/js/website_forum.js
+++ b/addons/website_forum/static/src/js/website_forum.js
@@ -148,7 +148,7 @@ function IsKarmaValid(eventNumber,minKarma){
         CKEDITOR.tools.callFunction(eventNumber,this);
         return false;
     } else {
-        alert("Sorry you need more than 30 Karma.");
+        alert("Sorry you need more than " + minKarma + " Karma.");
     }
 }
 
diff --git a/addons/website_mail/controllers/main.py b/addons/website_mail/controllers/main.py
index c0592000208fcea8db55aff03bd34636717ec1e3..18bddd09a5810371f640e09908c0a089aab3b0ed 100644
--- a/addons/website_mail/controllers/main.py
+++ b/addons/website_mail/controllers/main.py
@@ -92,9 +92,8 @@ class WebsiteMail(http.Controller):
                 values['is_follower'] = len(
                     request.registry['mail.followers'].search(
                         cr, SUPERUSER_ID, [
-                            ('res_model', '=', 'mail.group'),
+                            ('res_model', '=', model),
                             ('res_id', '=', obj_ids[0]),
                             ('partner_id', '=', partner_id.id)
                         ], context=context)) == 1
-
         return values
diff --git a/addons/website_mail/static/src/js/follow.js b/addons/website_mail/static/src/js/follow.js
new file mode 100644
index 0000000000000000000000000000000000000000..18c36378328a0e38e4212b07c200f0f54897a1b3
--- /dev/null
+++ b/addons/website_mail/static/src/js/follow.js
@@ -0,0 +1,68 @@
+(function () {
+    'use strict';
+
+    var website = openerp.website;
+
+    website.snippet.animationRegistry.follow = website.snippet.Animation.extend({
+        selector: ".js_follow",
+        start: function (editable_mode) {
+            var self = this;
+            this.is_user = false;
+
+            openerp.jsonRpc('/website_mail/is_follower', 'call', {
+                model: this.$target.data('object'),
+                id: this.$target.data('id'),
+            }).always(function (data) {
+                self.is_user = data.is_user;
+                self.email = data.email;
+                self.toggle_subscription(data.is_follower, data.email);
+                self.$target.removeClass("hidden");
+            });
+
+            // not if editable mode to allow designer to edit alert field
+            if (!editable_mode) {
+                $('.js_follow > .alert').addClass("hidden");
+                $('.js_follow > .input-group-btn.hidden').removeClass("hidden");
+                this.$target.find('.js_follow_btn, .js_unfollow_btn').on('click', function (event) {
+                    event.preventDefault();
+                    self.on_click();
+                });
+            }
+            return;
+        },
+        on_click: function () {
+            var self = this;
+            var $email = this.$target.find(".js_follow_email");
+
+            if ($email.length && !$email.val().match(/.+@.+/)) {
+                this.$target.addClass('has-error');
+                return false;
+            }
+            this.$target.removeClass('has-error');
+
+            openerp.jsonRpc('/website_mail/follow', 'call', {
+                'id': +this.$target.data('id'),
+                'object': this.$target.data('object'),
+                'message_is_follower': this.$target.attr("data-follow") || "off",
+                'email': $email.length ? $email.val() : false,
+            }).then(function (follow) {
+                self.toggle_subscription(follow, self.email);
+            });
+        },
+        toggle_subscription: function(follow, email) {
+            console.log(follow, email);
+            if (follow) {
+                this.$target.find(".js_follow_btn").addClass("hidden");
+                this.$target.find(".js_unfollow_btn").removeClass("hidden");
+            }
+            else {
+                this.$target.find(".js_follow_btn").removeClass("hidden");
+                this.$target.find(".js_unfollow_btn").addClass("hidden");
+            }
+            this.$target.find('input.js_follow_email')
+                .val(email ? email : "")
+                .attr("disabled", follow || (email.length && this.is_user) ? "disabled" : false);
+            this.$target.attr("data-follow", follow ? 'on' : 'off');
+        },
+    });
+})();
diff --git a/addons/website_mail/views/website_mail.xml b/addons/website_mail/views/website_mail.xml
index 2b2bbea7e5f555ccbbe1a8b9984fc53a097ba73c..0ccae00e21df82006c6ee16e791042b77a5bb611 100644
--- a/addons/website_mail/views/website_mail.xml
+++ b/addons/website_mail/views/website_mail.xml
@@ -21,6 +21,7 @@
 
 <template id="head" inherit_id="website.assets_frontend" name="Mail customization">
     <xpath expr="/t" position="inside">
+        <script type="text/javascript" src="/website_mail/static/src/js/follow.js"></script>
         <link rel='stylesheet' href='/website_mail/static/src/css/website_mail.css'/>
     </xpath>
 </template>
diff --git a/addons/website_mail_group/__init__.py b/addons/website_mail_group/__init__.py
index ee5959455ad41d51fbaff9d2ddcb980a25b622f2..9f86759e32be55377b85238f6ad6f71794bd4992 100644
--- a/addons/website_mail_group/__init__.py
+++ b/addons/website_mail_group/__init__.py
@@ -1 +1,2 @@
 import controllers
+import models
diff --git a/addons/website_mail_group/controllers/main.py b/addons/website_mail_group/controllers/main.py
index a28b808d9f2f9e28138568217fe812511d6cc6cf..59478b37fa3c16e2d5b72e703633fe0111b9f923 100644
--- a/addons/website_mail_group/controllers/main.py
+++ b/addons/website_mail_group/controllers/main.py
@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 
 import datetime
+from dateutil import relativedelta
 
-from openerp import tools
+from openerp import tools, SUPERUSER_ID
 from openerp.addons.web import http
 from openerp.addons.website.models.website import slug
 from openerp.addons.web.http import request
@@ -28,12 +29,23 @@ class MailGroup(http.Controller):
     def view(self, **post):
         cr, uid, context = request.cr, request.uid, request.context
         group_obj = request.registry.get('mail.group')
+        mail_message_obj = request.registry.get('mail.message')
         group_ids = group_obj.search(cr, uid, [('alias_id', '!=', False), ('alias_id.alias_name', '!=', False)], context=context)
-        values = {'groups': group_obj.browse(cr, uid, group_ids, context)}
+        groups = group_obj.browse(cr, uid, group_ids, context)
+        # compute statistics
+        month_date = datetime.datetime.today() - relativedelta.relativedelta(months=1)
+        group_data = dict.fromkeys(group_ids, dict())
+        for group in groups:
+            group_data[group.id]['monthly_message_nbr'] = mail_message_obj.search(
+                cr, SUPERUSER_ID,
+                [('model', '=', 'mail.group'), ('res_id', '=', group.id), ('date', '>=', month_date.strftime(tools.DEFAULT_SERVER_DATETIME_FORMAT))],
+                count=True, context=context)
+        values = {'groups': groups, 'group_data': group_data}
         return request.website.render('website_mail_group.mail_groups', values)
 
     @http.route(["/groups/subscription/"], type='json', auth="user")
     def subscription(self, group_id=0, action=False, **post):
+        """ TDE FIXME: seems dead code """
         cr, uid, context = request.cr, request.uid, request.context
         group_obj = request.registry.get('mail.group')
         if action:
diff --git a/addons/website_mail_group/models/__init__.py b/addons/website_mail_group/models/__init__.py
new file mode 100644
index 0000000000000000000000000000000000000000..ea8be51acde7bdf1ff8f83ee175090181fed7fed
--- /dev/null
+++ b/addons/website_mail_group/models/__init__.py
@@ -0,0 +1 @@
+import mail_group
diff --git a/addons/website_mail_group/models/mail_group.py b/addons/website_mail_group/models/mail_group.py
new file mode 100644
index 0000000000000000000000000000000000000000..804785b66f9eacb432a31b93e1641c6a517ba41a
--- /dev/null
+++ b/addons/website_mail_group/models/mail_group.py
@@ -0,0 +1,18 @@
+# -*- coding: utf-8 -*-
+
+from openerp.osv import osv
+
+
+class MailGroup(osv.Model):
+    _inherit = 'mail.group'
+
+    def message_get_email_values(self, cr, uid, id, notif_mail=None, context=None):
+        res = super(MailGroup, self).message_get_email_values(cr, uid, id, notif_mail=notif_mail, context=context)
+        group = self.browse(cr, uid, id, context=context)
+        base_url = self.pool['ir.config_parameter'].get_param(cr, uid, 'web.base.url')
+        res['headers'].update({
+            'List-Archive': '<%s/groups/%s>' % (base_url, group.id),
+            'List-Subscribe': '<%s/groups>' % (base_url),
+            'List-Unsubscribe': '<%s/groups>' % (base_url),
+        })
+        return res
diff --git a/addons/website_mail_group/static/src/js/website_mail_group.snippet.js b/addons/website_mail_group/static/src/js/website_mail_group.snippet.js
index fd6a2d38cfacba932981073b0ec444c171caad54..8ad4f9ba796e4ad3fcdae7f01df72fed7243ec54 100644
--- a/addons/website_mail_group/static/src/js/website_mail_group.snippet.js
+++ b/addons/website_mail_group/static/src/js/website_mail_group.snippet.js
@@ -3,8 +3,8 @@
 
     var website = openerp.website;
 
-    website.snippet.animationRegistry.follow = website.snippet.Animation.extend({
-        selector: ".js_follow",
+    website.snippet.animationRegistry.follow_alias = website.snippet.Animation.extend({
+        selector: ".js_follow_alias",
         start: function (editable_mode) {
             var self = this;
             this.is_user = false;
@@ -23,8 +23,8 @@
 
             // not if editable mode to allow designer to edit alert field
             if (!editable_mode) {
-                $('.js_follow > .alert').addClass("hidden");
-                $('.js_follow > .input-group-btn.hidden').removeClass("hidden");
+                $('.js_follow_alias > .alert').addClass("hidden");
+                $('.js_follow_alias > .input-group-btn.hidden').removeClass("hidden");
                 this.$target.find('.js_follow_btn, .js_unfollow_btn').on('click', function (event) {
                     event.preventDefault();
                     self.on_click();
diff --git a/addons/website_mail_group/views/snippets.xml b/addons/website_mail_group/views/snippets.xml
index 452f59f490248519433dd5c4ffedea8f6df507dd..95935cd49988c2e3b962b1330933c688829006ae 100644
--- a/addons/website_mail_group/views/snippets.xml
+++ b/addons/website_mail_group/views/snippets.xml
@@ -11,7 +11,7 @@
                 <span class="oe_snippet_thumbnail_title">Discussion Group</span>
             </div>
 
-            <div class="oe_snippet_body js_follow"
+            <div class="oe_snippet_body js_follow_alias"
                       data-id="0"
                       data-object="mail.group"
                       data-follow="off">
@@ -38,7 +38,7 @@
 
     <xpath expr="//div[@id='snippet_options']" position="inside">
         <div data-snippet-option-id='subscribe'
-            data-selector=".js_follow"
+            data-selector=".js_follow_alias"
             data-selector-siblings="p, h1, h2, h3, blockquote, .well, .panel"
            >
             <li>
diff --git a/addons/website_mail_group/views/website_mail_group.xml b/addons/website_mail_group/views/website_mail_group.xml
index 8ecbd6c84ed2bbc932d08822d214b60f0cee88c5..a10650277483e647197913f4a20f91a2d3c13ef9 100644
--- a/addons/website_mail_group/views/website_mail_group.xml
+++ b/addons/website_mail_group/views/website_mail_group.xml
@@ -45,7 +45,7 @@
                 </div>
                 <div class="col-md-2">
                     <i class='fa fa-user'/> <t t-esc="len(group.message_follower_ids)"/> participants<br />
-                    <i class='fa fa-envelope-o'/> <t t-esc="len(group.message_ids)"/> messages
+                    <i class='fa fa-envelope-o'/> <t t-raw="group_data[group.id]['monthly_message_nbr']"/> messages / month
                 </div>
                 <div class="col-md-3">
                     <t t-call="website_mail.follow"><t t-set="object" t-value="group"/></t>
diff --git a/addons/website_sale/controllers/main.py b/addons/website_sale/controllers/main.py
index 23225186dc8b392b9e294b7f19fb9529d2b81bbd..a6bcd15f9e503fe96e7f65166d806f1efa5b7f3c 100644
--- a/addons/website_sale/controllers/main.py
+++ b/addons/website_sale/controllers/main.py
@@ -55,7 +55,7 @@ class table_compute(object):
                     self.table[(pos/PPR)+y2][(pos%PPR)+x2] = False
             self.table[pos/PPR][pos%PPR] = {
                 'product': p, 'x':x, 'y': y,
-                'class': " ".join(map(lambda x: x.html_class, p.website_style_ids))
+                'class': " ".join(map(lambda x: x.html_class or '', p.website_style_ids))
             }
             if index<=PPG:
                 maxy=max(maxy,y+(pos/PPR))
@@ -169,7 +169,7 @@ class website_sale(http.Controller):
 
         values = {
             'search': search,
-            'category': category and int(category),
+            'category': category,
             'attrib_values': attrib_values,
             'attrib_set': attrib_set,
             'pager': pager,
diff --git a/addons/website_sale/views/templates.xml b/addons/website_sale/views/templates.xml
index c43363b863e2b590857cda1d4fbd6e7d69062ef5..fb5a83de1bd7130fba8b35516798a88dac42e7f2 100644
--- a/addons/website_sale/views/templates.xml
+++ b/addons/website_sale/views/templates.xml
@@ -240,7 +240,7 @@
 <!-- Add to cart button-->
 
 <template id="categories_recursive" name="Category list">
-  <li t-att-class="'active' if c.id == category else ''">
+  <li t-att-class="'active' if c.id == int(category or 0) else ''">
       <a t-att-href="keep('/shop/category/' + slug(c), category=0)" t-field="c.name"></a>
       <ul t-if="c.child_id" class="nav nav-pills nav-stacked nav-hierarchy">
           <t t-foreach="c.child_id" t-as="c">
diff --git a/openerp/addons/base/ir/ir_actions.py b/openerp/addons/base/ir/ir_actions.py
index d6d96737ca3e11046bf3f34588027a4c40c84608..3c84533e71631959f4d68f1f32c03049412344f4 100644
--- a/openerp/addons/base/ir/ir_actions.py
+++ b/openerp/addons/base/ir/ir_actions.py
@@ -491,6 +491,8 @@ class ir_actions_server(osv.osv):
                                    "based on the sequence. Low number means high priority."),
         'model_id': fields.many2one('ir.model', 'Base Model', required=True, ondelete='cascade',
                                     help="Base model on which the server action runs."),
+        'model_name': fields.related('model_id', 'model', type='char',
+                                     string='Model Name', readonly=True),
         'menu_ir_values_id': fields.many2one('ir.values', 'More Menu entry', readonly=True,
                                              help='More menu entry.'),
         # Client Action
@@ -650,6 +652,10 @@ class ir_actions_server(osv.osv):
             'wkf_field_id': False,
             'crud_model_id': model_id,
         }
+
+        if model_id:
+            values['model_name'] = self.pool.get('ir.model').browse(cr, uid, model_id, context).model
+
         return {'value': values}
 
     def on_change_wkf_wonfig(self, cr, uid, ids, use_relational_model, wkf_field_id, wkf_model_id, model_id, context=None):
@@ -753,6 +759,7 @@ class ir_actions_server(osv.osv):
         crud_model_name = False
         if crud_model_id:
             crud_model_name = self.pool.get('ir.model').browse(cr, uid, crud_model_id, context).model
+        
         values = {'link_field_id': False, 'crud_model_name': crud_model_name}
         return {'value': values}
 
diff --git a/openerp/addons/base/ir/ir_actions.xml b/openerp/addons/base/ir/ir_actions.xml
index 6cb498e9ebf1d1c180d5f29a6ea6744af1493238..2aa520d7c553e5354ed52abdb59ef66c8569cc75 100644
--- a/openerp/addons/base/ir/ir_actions.xml
+++ b/openerp/addons/base/ir/ir_actions.xml
@@ -349,8 +349,9 @@
                                             Check to attach the newly created record to the record on which the server action runs.
                                         </p>
                                         <group>
+                                            <field name="model_name" invisible="1"/>
                                             <field name="link_field_id"
-                                                    domain="[('model_id', '=', model_id), ('relation', '=', crud_model_name), ('ttype', 'in', ['many2one'])]"
+                                                    domain="[('model_id', '=', crud_model_id), ('relation', '=', model_name), ('ttype', 'in', ['many2one'])]"
                                                     attrs="{'required': [('state', '=', 'object_create'), ('link_new_record', '=', True)],
                                                             'invisible': ['|', ('state', '!=', 'object_create'), ('link_new_record', '=', False)]}"/>
                                         </group>
diff --git a/openerp/osv/fields.py b/openerp/osv/fields.py
index 57961a1d32cffd95fe68de99f8b04728d6498fa7..0eae4185aeb31a14d08098d83be7c66b167ac6eb 100644
--- a/openerp/osv/fields.py
+++ b/openerp/osv/fields.py
@@ -561,7 +561,7 @@ class many2one(_column):
         # we use uid=1 because the visibility of a many2one field value (just id and name)
         # must be the access right of the parent form and not the linked object itself.
         records = dict(obj.name_get(cr, SUPERUSER_ID,
-                                    list(set([x for x in res.values() if isinstance(x, (int,long))])),
+                                    list(set([x for x in res.values() if x and isinstance(x, (int,long))])),
                                     context=context))
         for id in res:
             if res[id] in records:
diff --git a/openerp/osv/orm.py b/openerp/osv/orm.py
index 0a23dda61e39eb3dbb0f9bb4c2238c5f07b10f0f..4289d6660f8a05455e2bf037e613674379ca2014 100644
--- a/openerp/osv/orm.py
+++ b/openerp/osv/orm.py
@@ -3348,6 +3348,8 @@ class BaseModel(object):
             return []
         if fields_to_read is None:
             fields_to_read = self._columns.keys()
+        else:
+            fields_to_read = list(set(fields_to_read))
 
         # all inherited fields + all non inherited fields for which the attribute whose name is in load is True
         fields_pre = [f for f in fields_to_read if
diff --git a/addons/mail/mail_followers.py b/addons/mail/mail_followers.py
index ffc49414b2e33368a70272f02ffecf8c14c010fa..e4028111a733a84db2d0648eae9502961f1e5e82 100644
--- a/addons/mail/mail_followers.py
+++ b/addons/mail/mail_followers.py
@@ -176,7 +176,7 @@ class mail_notification(osv.Model):
         references = message.parent_id.message_id if message.parent_id else False
 
         # create email values
-        max_recipients = 100
+        max_recipients = 50
         chunks = [email_pids[x:x + max_recipients] for x in xrange(0, len(email_pids), max_recipients)]
         email_ids = []
         for chunk in chunks:
@@ -188,7 +188,7 @@ class mail_notification(osv.Model):
                 'references': references,
             }
             email_ids.append(self.pool.get('mail.mail').create(cr, uid, mail_values, context=context))
-        if force_send and len(chunks) < 6:  # for more than 500 followers, use the queue system
+        if force_send and len(chunks) < 2:  # for more than 50 followers, use the queue system
             self.pool.get('mail.mail').send(cr, uid, email_ids, context=context)
         return True
 